package com.sun.jndi.ldap.pool;

import java.util.Collections;
import java.util.LinkedList;
import com.sun.jndi.ldap.LdapPoolManager;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.ArrayList;
import javax.naming.NamingException;
import java.util.WeakHashMap;
import java.util.Map;
import java.lang.ref.Reference;
import java.util.Collection;
import java.lang.ref.ReferenceQueue;

public final class Pool
{
    static final boolean debug;
    private static final ReferenceQueue<ConnectionsRef> queue;
    private static final Collection<Reference<ConnectionsRef>> weakRefs;
    private final int maxSize;
    private final int prefSize;
    private final int initSize;
    private final Map<Object, ConnectionsRef> map;
    
    public Pool(int initSize, int prefSize, int maxSize) {
        this.map = new WeakHashMap();
        this.prefSize = prefSize;
        this.maxSize = maxSize;
        this.initSize = initSize;
    }
    
    public PooledConnection getPooledConnection(Object o, long n, PooledConnectionFactory pooledConnectionFactory) throws NamingException {
        this.d("get(): ", o);
        if (Pool.debug) {
            synchronized (this.map) {
                this.d("size: ", this.map.size());
            }
        }
        expungeStaleConnections();
        Map<Object, Object> connections;
        synchronized (this.map) {
            connections = this.getConnections(o);
            if (connections == null) {
                this.d("get(): creating new connections list for ", o);
                connections = new Connections(o, this.initSize, this.prefSize, this.maxSize, pooledConnectionFactory);
                ConnectionsRef connectionsRef = new ConnectionsRef(connections);
                this.map.put(o, connectionsRef);
                Pool.weakRefs.add(new ConnectionsWeakRef(connectionsRef, Pool.queue));
            }
            this.d("get(): size after: ", this.map.size());
        }
        return connections.get(n, pooledConnectionFactory);
    }
    
    private Connections getConnections(Object o) {
        ConnectionsRef connectionsRef = (ConnectionsRef)this.map.get(o);
        return (connectionsRef != null) ? connectionsRef.getConnections() : null;
    }
    
    public void expire(long n) {
        ArrayList list;
        synchronized (this.map) {
            list = new ArrayList(this.map.values());
        }
        ArrayList<ConnectionsRef> list2 = new ArrayList();
        for (ConnectionsRef e : list) {
            Connections connections = e.getConnections();
            if (connections.expire(n)) {
                this.d("expire(): removing ", connections);
                list2.add(e);
            }
            else {
                continue;
            }
        }
        synchronized (this.map) {
            this.map.values().removeAll(list2);
        }
        expungeStaleConnections();
    }
    
    private static void expungeStaleConnections() {
        ConnectionsWeakRef connectionsWeakRef;
        while ((connectionsWeakRef = (ConnectionsWeakRef)Pool.queue.poll()) != null) {
            Connections connections = connectionsWeakRef.getConnections();
            if (Pool.debug) {
                System.err.println("weak reference cleanup: Closing Connections:" + connections);
            }
            connections.close();
            Pool.weakRefs.remove(connectionsWeakRef);
            connectionsWeakRef.clear();
        }
    }
    
    public void showStats(PrintStream printStream) {
        printStream.println("===== Pool start ======================");
        printStream.println("maximum pool size: " + this.maxSize);
        printStream.println("preferred pool size: " + this.prefSize);
        printStream.println("initial pool size: " + this.initSize);
        synchronized (this.map) {
            printStream.println("current pool size: " + this.map.size());
            for (Map.Entry entry : this.map.entrySet()) {
                printStream.println("   " + entry.getKey() + ":" + ((ConnectionsRef)entry.getValue()).getConnections().getStats());
            }
        }
        printStream.println("====== Pool end =====================");
    }
    
    @Override
    public String toString() {
        synchronized (this.map) {
            return super.toString() + " " + this.map.toString();
        }
    }
    
    private void d(String str, int i) {
        if (Pool.debug) {
            System.err.println(this + "." + str + i);
        }
    }
    
    private void d(String str, Object obj) {
        if (Pool.debug) {
            System.err.println(this + "." + str + obj);
        }
    }
    
    static {
        debug = LdapPoolManager.debug;
        queue = new ReferenceQueue();
        weakRefs = Collections.synchronizedList(new LinkedList());
    }
}
